@@ -1,12 +1,13 @@
 package MooseX::Traits;
 use Moose::Role;
 
-our $VERSION   = '0.02';
+our $VERSION   = '0.03';
 our $AUTHORITY = 'id:JROCKWAY';
 
 has '_trait_namespace' => (
     # no accessors or init_arg
-    isa => 'Str',
+    init_arg => undef,
+    isa      => 'Str',
 );
 
 # dont pollute the consuming class with methods they don't want
@@ -28,21 +29,22 @@ my $transform_trait = sub {
 
 sub new_with_traits {
     my ($class, %args) = @_;
-    
-    my @traits = 
-      grep { Class::MOP::load_class($_); 1 }
-        map { $class->$transform_trait($_) } @{ delete $args{traits} || [] };
 
-    my @anon_args = (
-        superclasses => [$class->meta->name],
-        cache        => 1,
-    );
-    push @anon_args, roles => [@traits] if @traits;
+    if (my $traits = delete $args{traits}) {
 
-    my $metaclass = $class->meta->meta->name;    
-    my $new_class = $metaclass->create_anon_class(@anon_args);
+        Class::MOP::load_class($_)
+            for map { $_ = $class->$transform_trait($_) } @$traits;
 
-    return $new_class->name->new(%args);
+        my $meta = $class->meta->create_anon_class(
+            superclasses => [ blessed($class) || $class ],
+            roles        => $traits,
+            cache        => 1,
+        );
+        $meta->add_method('meta' => sub { $meta });
+        $class = $meta->name;
+    }
+
+    return $class->new(%args);
 }
 
 1;
@@ -89,15 +91,18 @@ in one go, cache the resulting class (for efficiency), and return a
 new instance.  Arguments meant to initialize the applied roles'
 attributes can also be passed to the constructor.
 
+=head1 METHODS
+
+=over 4
+
+=item B<new_with_traits (%args, traits => \@traits)>
+
+=back
+
 =head1 ATTRIBUTES YOUR CLASS GETS
 
 This role will add the following attributes to the consuming class.
 
-=head2 traits
-
-The list of traits to be applied.  Acts as an C<init_arg> and a C<ro>
-accessor.
-
 =head2 _trait_namespace
 
 You can override the value of this attribute with C<default> to
@@ -118,7 +123,7 @@ Example:
   use Moose;
   with 'MooseX::Traits';
   has '+_trait_namespace' => ( default => 'Another' );
- 
+
   my $instance = Another::Class->new_with_traits(
       traits => ['Trait'], # "Another::Trait", not "Trait"
       bar    => 'bar',
@@ -136,6 +141,8 @@ Example:
 
 Jonathan Rockway C<< <jrockway@cpan.org> >>
 
+Stevan Little C<< <stevan.little@iinteractive.com> >>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2008 Infinity Interactive, Inc.
